'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = reducer;

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

var _ = require('lodash/fp');

var LOAD = 'tree/LOAD';
var LOAD_SUCCESS = 'tree/LOAD_SUCCESS';
var LOAD_FAIL = 'tree/LOAD_FAIL';
var TOGGLE_CHECKED = 'tree/TOGGLE_CHECKED';
var TOGGLE_COLLAPSED = 'tree/TOGGLE_COLLAPSED';
var SET_CURRENT_NODE = 'tree/SET_CURRENT_NODE';
var REMOVE = 'tree/REMOVE';

var initialState = {
  current: '',
  error: null,
  nodes: [],
  loading: false,
  loaded: false
};

var getSelector = function getSelector(nodeId) {
  return nodeId.split('.').join('.children.');
};

var prepareNodes = function prepareNodes(nodes, initOptions) {
  var collapsed = initOptions.collapsed === undefined ? true : initOptions.collapsed;
  var checked = initOptions.checked === undefined ? false : initOptions.checked;

  var prepareNode = function prepareNode(node) {
    var hasChildren = _.get('children', node);
    node = _.set('checked', checked, node);
    node = _.set('current', false, node);

    if (hasChildren) {
      node = _.set('collapsed', collapsed, node);
      node = _.set('loaded', false, node);
      node = _.set('loading', false, node);
      node = _.set('children', _.map(function (childNode) {
        return prepareNode(childNode);
      }, node.children), node);
    }

    return node;
  };

  return _.map(function (node) {
    return prepareNode(node);
  }, nodes);
};

function reducer() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? initialState : arguments[0];
  var action = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  switch (action.type) {
    case LOAD:
      console.log('LOAD:', action);
      return _extends({}, state, {
        loading: true
      });
    case LOAD_SUCCESS:
      console.log('LOAD_SUCCESS; nodeId: ' + action.nodeId + '; result: ', action.result);

      var selector = getSelector(action.nodeId ? '' + action.nodeId : '');
      var preparedChildren = prepareNodes(action.result, action.initOptions);
      var preparedNodes = _.set(selector + '.loaded', true, state.nodes);
      preparedNodes = _.set(selector + '.children', preparedChildren, preparedNodes);
      console.log('prepared', preparedNodes);
      return _extends({}, state, {
        nodes: state.nodes.length !== 0 ? preparedNodes : preparedChildren,
        loaded: true,
        loading: false,
        error: null
      });
    case LOAD_FAIL:
      console.log('LOAD_FAIL', action.nodeId, action.error);
      return _extends({}, state, {
        error: action.error
      });
    case TOGGLE_CHECKED:
      var checked = _.get(getSelector(action.nodeId) + '.checked', state.nodes);
      return _extends({}, state, {
        nodes: _.set(getSelector(action.nodeId) + '.checked', !checked, state.nodes)
      });
    case TOGGLE_COLLAPSED:
      var collapsed = _.get(getSelector(action.nodeId) + '.collapsed', state.nodes);
      return _extends({}, state, {
        nodes: _.set(getSelector(action.nodeId) + '.collapsed', !collapsed, state.nodes)
      });
    case SET_CURRENT_NODE:
      var nodes = _.set(getSelector(state.current) + '.current', false, state.nodes);
      nodes = _.set(getSelector(action.nodeId) + '.current', true, nodes);
      return _extends({}, state, {
        current: action.nodeId,
        nodes: nodes
      });
    case REMOVE:
      return _extends({}, state, {
        nodes: action.nodes
      });
    default:
      return state;
  }
};

var load = exports.load = function load(nodeId, initOptions) {
  initOptions = initOptions ? initOptions : {};
  return {
    types: [LOAD, LOAD_SUCCESS, LOAD_FAIL],
    nodeId: nodeId,
    initOptions: initOptions,
    promise: function promise(client) {
      return client.get('/treeview/load?nodeId=' + (nodeId ? nodeId : ''));
    }
  };
};

var toggleChecked = exports.toggleChecked = function toggleChecked(nodeId) {
  return { type: TOGGLE_CHECKED, nodeId: nodeId };
};

var toggleCollapsed = exports.toggleCollapsed = function toggleCollapsed(nodeId, loaded) {
  if (loaded) {
    return { type: TOGGLE_COLLAPSED, nodeId: nodeId, loaded: loaded };
  } else {
    return function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(dispatch) {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                dispatch(toggleCollapsed(nodeId, true));
                dispatch(load(nodeId));

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, undefined);
      })),
          _this = undefined;

      return function (_x3) {
        return ref.apply(_this, arguments);
      };
    }();
  }
};

var setCurrentNode = exports.setCurrentNode = function setCurrentNode(nodeId) {
  return { type: SET_CURRENT_NODE, nodeId: nodeId };
};

var remove = exports.remove = function remove(nodes) {
  return { type: REMOVE, nodesId: nodesId };
};